#!/usr/bin/env python3

# Author::    Justin Flannery  (mailto:juftin@juftin.com)

"""
Storage Containers for the Application
"""

from datetime import datetime
import logging
from typing import NamedTuple, Tuple

logger = logging.getLogger(__name__)


class SearchWindow(NamedTuple):
    """
    Search Window for Campsite Search
    """
    start_date: datetime
    end_date: datetime


class AvailableCampsite(NamedTuple):
    """
    Campsite Storage, this container should be universal regardless of
    API Provider
    """
    campsite_id: int
    booking_date: datetime
    booking_end_date: datetime
    booking_nights: int
    campsite_site_name: str
    campsite_loop_name: str
    campsite_type: str
    campsite_occupancy: Tuple[int, int]
    campsite_use_type: str
    availability_status: str
    recreation_area: str
    recreation_area_id: int
    facility_name: str
    facility_id: int
    booking_url: str


class CampgroundFacility(NamedTuple):
    """
    Campground Facility Data Storage
    """
    facility_name: str
    recreation_area: str
    facility_id: int
    recreation_area_id: int


class RecreationArea(NamedTuple):
    """
    Recreation Area Data Storage
    """
    recreation_area: str
    recreation_area_id: int
    recreation_area_location: str



#!/usr/bin/env python3

# Author::    Justin Flannery  (mailto:juftin@juftin.com)

"""
Notifications __init__ file
"""

from .email_notifications import EmailNotifications
from .pushbullet import PushbulletNotifications
from .pushover import PushoverNotifications
from .silent_notifications import SilentNotifications

CAMPSITE_NOTIFICATIONS: dict = {
    "pushover": PushoverNotifications,
    "email": EmailNotifications,
    "silent": SilentNotifications,
    "pushbullet": PushbulletNotifications
}



#!/usr/bin/env python3

# Author::    Justin Flannery  (mailto:juftin@juftin.com)

"""
Camply Configuration Script
"""

from collections import OrderedDict
from datetime import datetime
import logging
from os.path import isfile
from time import sleep

from camply.config import FileConfig

logger = logging.getLogger(__name__)


def get_log_input(message: str):
    """
    Create a log message with a nice log format :)
    Parameters
    ----------
    message: str
        The message you'd like to print before getting input
    """
    datetime_string = datetime.utcnow().strftime('%Y-%m-%d %H:%M:%S,%f')[:-3]
    input_string = f"{datetime_string} [   INPUT]: {message} : "
    value = input(input_string)
    return value


def double_check(message: str) -> bool:
    """
    Double check if a step should be taken within CLI

    Parameters
    ----------
    message: str
        Message to log in interactive shell
    """
    operation_eval = True
    while operation_eval:
        first_confirmation = get_log_input(message=message)
        if first_confirmation.lower() == "y":
            second_confirmation = get_log_input("Are you sure? (y/n)")
            if second_confirmation.lower() != "y":
                logging.info("Okay, skipping")
                return False
            elif second_confirmation.lower() == "y":
                return True
        elif first_confirmation.lower() == "n":
            logging.info("Okay, skipping")
            return False
        else:
            logging.warning("Make sure to enter a 'y' or 'n'")
            operation_eval = True


def check_dot_camply_file() -> bool:
    """
    Check to see if the `.camply` file already exists, and return the
    file existence status

    Returns
    -------
    bool
    """
    if isfile(FileConfig.DOT_CAMPLY_FILE) is True:
        logger.info("Skipping configuration. `.camply` file already exists: "
                    f"{FileConfig.DOT_CAMPLY_FILE}")
        return True
    else:
        return False


def generate_configuration() -> OrderedDict:
    """
    Generate the Camply Configuration Config

    Returns
    -------
    OrderedDict
        Dict of configuration values
    """
    config_dict = FileConfig.DOT_CAMPLY_FIELDS.copy()
    for field, field_dict in config_dict.items():
        default_value = field_dict["default"]
        field_note = field_dict['notes']
        if field_note is not None:
            logger.info(f"{field}: {field_note}")
        message = f"Enter value for `{field}`"
        if default_value != "":
            message += f" (default: `{default_value}`)"
        logged_input = get_log_input(message=message).strip()
        config_value = logged_input if logged_input != '' else default_value
        config_dict[field] = config_value
    return config_dict


def write_config_to_file(config_dict: OrderedDict) -> None:
    """
    Write the Configuration Object to a file

    Parameters
    ----------
    config_dict : OrderedDict
        Configuration Object
    """
    string_list = [
        "# CAMPLY CONFIGURATION FILE. ",
        "# SEE https://github.com/juftin/camply/blob/main/docs/examples/example.camply ""FOR MORE DETAILS",
        ""
    ]
    for config_key, config_value in config_dict.items():
        string_list.append(f'{config_key}="{config_value}"')
    string_list.append("")
    with open(FileConfig.DOT_CAMPLY_FILE, "w") as file_object:
        file_object.write("\n".join(string_list))
        file_object.seek(0)


def generate_dot_camply_file():
    """
    Perform the larger Dot Camply File Generation
    """
    logger.info("Running camply configuration.")
    logger.info("This process generates a configuration file "
                "(https://github.com/juftin/camply/blob/main/example.camply)")
    logger.info("Do not include quotes around values")
    logger.info("To skip a configuration field or keep it as default, just press <Enter>.")
    sleep(1.5)
    if isfile(FileConfig.DOT_CAMPLY_FILE):
        logger.warning(f".camply file already exists on this machine: {FileConfig.DOT_CAMPLY_FILE}")
        overwrite = double_check("Would you like to overwrite your `.camply` "
                                 "configuration file? (y/n)")
        if overwrite is False:
            exit(0)
    config = generate_configuration()
    if double_check(f"Are you ready to publish this to a file at {FileConfig.DOT_CAMPLY_FILE}"):
        write_config_to_file(config_dict=config)
        logger.info(f"`.camply` file written to machine: {FileConfig.DOT_CAMPLY_FILE}")



#!/usr/bin/env python3

# Author::    Justin Flannery  (mailto:juftin@juftin.com)

"""
Config __init__ file
"""

from .api_config import RecreationBookingConfig, RIDBConfig, STANDARD_HEADERS, USER_AGENTS
from .cli_config import CommandLineConfig
from .data_columns import CampsiteContainerFields, DataColumns
from .file_config import FileConfig
from .notification_config import EmailConfig, PushbulletConfig, PushoverConfig
from .search_config import SearchConfig
from .yellowstone_config import YellowstoneConfig



#!/usr/bin/env python3

# Author::    Justin Flannery  (mailto:juftin@juftin.com)

"""
Command Line Configuration
"""

import logging
from os import getenv

from dotenv import load_dotenv

from camply import __version__ as camply_version
from camply.config.file_config import FileConfig
from camply.config.search_config import SearchConfig

load_dotenv(FileConfig.DOT_CAMPLY_FILE, override=False)


class CommandLineActions:
    """
    ArgParse Actions
    """
    VERSION: str = "version"
    STORE: str = "store"
    STORE_TRUE: str = "store_true"
    APPEND: str = "append"


class CommandLineArguments:
    """
    Argument Config
    """
    VERSION_ARGUMENT: str = "--version"

    STATE_ARGUMENT: str = "--state"
    STATE_DESTINATION: str = "state"
    STATE_HELP: str = "Filter by US state code."

    REC_AREA_ID_ARGUMENT: str = "--rec-area"
    REC_AREA_ID_DESTINATION: str = "recreation_area_id"
    REC_AREA_ID_HELP: str = "Add Recreation Areas (comprised of campgrounds) by ID."

    CAMPGROUND_LIST_ARGUMENT: str = "--campground"
    CAMPGROUND_LIST_DESTINATION: str = "campground_id"
    CAMPGROUND_LIST_HELP: str = "Add individual Campgrounds by ID."

    SEARCH_ARGUMENT: str = "--search"
    SEARCH_DESTINATION: str = "search"
    SEARCH_HELP: str = "Search for Campgrounds or Recreation Areas by search string."

    START_DATE_ARGUMENT: str = "--start-date"
    START_DATE_DESTINATION: str = "start_date"
    START_DATE_HELP: str = "(YYYY-MM-DD) Start of Search window. You will be arriving this day."

    END_DATE_ARGUMENT: str = "--end-date"
    END_DATE_DESTINATION: str = "end_date"
    END_DATE_HELP: str = "(YYYY-MM-DD) End of Search window. You will be checking out this day."

    WEEKENDS_ARGUMENT: str = "--weekends"
    WEEKENDS_DESTINATION: str = "weekends"
    WEEKENDS_HELP: str = "Only search for weekend bookings (Fri/Sat nights)."

    NIGHTS_ARGUMENT: str = "--nights"
    NIGHTS_DESTINATION: str = "nights"
    NIGHTS_DEFAULT: int = 1
    NIGHTS_HELP: str = ("Search for campsite stays with consecutive nights. "
                        "Defaults to 1 which returns all campsites found.")

    PROVIDER_ARGUMENT: str = "--provider"
    PROVIDER_DESTINATION: str = "provider"
    PROVIDER_DEFAULT: str = "RecreationDotGov"
    PROVIDER_HELP: str = ("Camping Search Provider. Options available are 'Yellowstone' and "
                          "'RecreationDotGov'. Defaults to 'RecreationDotGov', not case-sensitive.")

    CONTINUOUS_ARGUMENT: str = "--continuous"
    CONTINUOUS_DESTINATION: str = "continuous"
    CONTINUOUS_HELP: str = ("Continuously check for a campsite to become available, and quit once "
                            "at least one campsite is found.")

    POLLING_INTERVAL_ARGUMENT: str = "--polling-interval"
    POLLING_INTERVAL_DESTINATION: str = "polling_interval"
    POLLING_INTERVAL_DEFAULT: str = SearchConfig.RECOMMENDED_POLLING_INTERVAL
    POLLING_INTERVAL_HELP: str = ("If --continuous is activated, how often to wait in between "
                                  "checks (in minutes). Defaults to 10, cannot be less than 5.")

    NOTIFICATIONS_ARGUMENT: str = "--notifications"
    NOTIFICATIONS_DESTINATION: str = "notifications"
    NOTIFICATIONS_DEFAULT: str = "silent"
    NOTIFICATIONS_HELP: str = ("If --continuous is activated, types of notifications to receive. "
                               "Options available are 'email', "
                               "'pushover', 'pushbullet', or 'silent'. Defaults to 'silent' - "
                               "which just logs messages to console.")

    NOTIFY_FIRST_TRY_ARGUMENT: str = "--notify-first-try"
    NOTIFY_FIRST_TRY_DESTINATION: str = "notify_first_try"
    NOTIFY_FIRST_TRY_HELP: str = ("If --continuous is activated, whether to send all "
                                  "non-silent notifications if more than 5 matching "
                                  "campsites are found on the first try. Defaults to false which "
                                  "only sends the first 5.")

    SEARCH_FOREVER_ARGUMENT: str = "--search-forever"
    SEARCH_FOREVER_DESTINATION: str = "search_forever"
    SEARCH_FOREVER_HELP: str = ("If --continuous is activated, this method continues to search "
                                "after the first availability has been found. The one caveat is "
                                "that it will never notify about the same identical campsite for "
                                "the same booking date.")

    YAML_SEARCH_ARGUMENT: str = "--yml-config"
    YAML_SEARCH_DESTINATION: str = "yml_config"
    YAML_SEARCH_HELP: str = ("Rather than provide arguments to the command line utility, instead "
                             "pass a file path to a YAML configuration file. See the documentation "
                             "for more information on how to structure your configuration file.")


class CommandLineValidation:
    """
    Camply CLI Validation Config
    """
    ERROR_NO_ARGUMENT_FOUND: str = "You must provide an argument to the Camply CLI"
    ERROR_MESSAGE_RECREATION_AREA: str = ("You must add a --search or --state parameter to search "
                                          "for Recreation Areas.")
    ERROR_MESSAGE_CAMPGROUNDS: str = ("You must add a --search, --state, --campground, or "
                                      "--rec-area parameter to search for Campgrounds.")
    ERROR_MESSAGE_REC_DOT_GOV: str = ("To search for Recreation.gov Campsites you must provide "
                                      "either the --rec-area or the --campground parameters.")
    ERROR_MESSAGE_CAMPSITES: str = ("Campsite searches require the following mandatory search "
                                    "parameters.")


class CommandLineConfig(CommandLineActions, CommandLineArguments, CommandLineValidation):
    """
    Command Line String Arguments and Flags
    """

    CAMPLY_APP_NAME: str = "camply"
    CAMPLY_APP_VERSION: str = camply_version
    CAMPLY_DESCRIPTION: str = "camply, the campsite finder"
    CAMPLY_EXIT_MESSAGE: str = "Exiting camply 👋"

    CAMPLY_LONG_DESCRIPTION: str = ("Welcome to camply, the campsite finder. "
                                    "Finding reservations at sold out campgrounds can be "
                                    "tough. That's where camply comes in. It searches the APIs of "
                                    "booking services like https://recreation.gov (which indexes "
                                    "thousands of campgrounds across the USA) to continuously "
                                    "check for cancellations and availabilities to pop up. "
                                    "Once a campsite becomes available, camply sends you a "
                                    "notification to book your spot!")
    CAMPLY_EPILOG: str = "visit the camply documentation at https://github.com/juftin/camply"

    LOG_LEVEL = logging.getLevelName(getenv("LOG_LEVEL", "INFO").upper())

    COMMAND_DESTINATION: str = "command"

    COMMAND_RECREATION_AREA: str = "recreation-areas"
    COMMAND_RECREATION_AREA_HELP: str = "Search for Recreation Areas and list them"
    COMMAND_RECREATION_AREA_DESCRIPTION: str = ("Search for Recreation Areas and their IDs. "
                                                "Recreation Areas are places like National Parks "
                                                "and National Forests that can contain one or many "
                                                "campgrounds.")

    COMMAND_CAMPGROUNDS: str = "campgrounds"
    COMMAND_CAMPGROUNDS_HELP: str = ("Search for Campgrounds (inside of Recreation Areas) "
                                     "and list them")
    COMMAND_CAMPGROUNDS_DESCRIPTION: str = ("Search for Campgrounds and their IDs. "
                                            "Campgrounds are facilities inside of Recreation Areas "
                                            "that contain campsites. Most 'campgrounds' are areas "
                                            "made up of multiple campsites, others are "
                                            "facilities like fire towers or cabins that might only "
                                            "contain a single 'campsite' to book.")

    COMMAND_CAMPSITES: str = "campsites"
    COMMAND_CAMPSITES_HELP: str = "Find available Campsites using search criteria"
    COMMAND_CAMPSITES_DESCRIPTION: str = ("Search for a campsite within camply. Campsites are "
                                          "returned based on the search criteria provided. "
                                          "Campsites contain properties like booking date, site "
                                          "type (tent, RV, cabin, etc), capacity, price, and a "
                                          "link to make the booking. "
                                          "Required parameters include `--start-date`, "
                                          "`--end-date`, `--rec-area` / `--campground`. "
                                          "Constant searching functionality can be enabled with "
                                          " `--continuous` and notifications "
                                          "can be enabled using `--notifications`.")

    COMMAND_CONFIGURE: str = "configure"
    COMMAND_CONFIGURE_HELP: str = "Set up camply configuration file with an interactive console"
    COMMAND_CONFIGURE_DESCRIPTION: str = (
        "In order to send notifications through camply you must set up some authorization "
        "values. Whether you need to set up pushover notifications (push notifications on "
        "your phone, your pushover account can be set up at https://pushover.net) or Email "
        "messages, everything can be done through the configure command. The end result "
        "is a file called .camply in your home folder."
    )
    RECREATION_DOT_GOV: str = "RecreationDotGov"



#!/usr/bin/env python3

# Author::    Justin Flannery  (mailto:juftin@juftin.com)

"""
camply search __init__ file
"""

from typing import Dict

from .base_search import BaseCampingSearch
from .search_recreationdotgov import SearchRecreationDotGov
from .search_yellowstone import SearchYellowstone

CAMPSITE_SEARCH_PROVIDER: Dict[str, object] = {
    "RecreationDotGov": SearchRecreationDotGov,
    "Yellowstone": SearchYellowstone
}



#!/usr/bin/env python3

# Author::    Justin Flannery  (mailto:juftin@juftin.com)

"""
Camply Command Line Interface
"""

from argparse import ArgumentParser, Namespace
from datetime import datetime
import logging
from typing import Optional, Tuple

from camply.config import CommandLineConfig
from camply.containers import SearchWindow
from camply.providers import RecreationDotGov
from camply.search import CAMPSITE_SEARCH_PROVIDER
from camply.utils import configure_camply, yaml_utils
from camply.utils import log_camply

logging.Logger.camply = log_camply
logger = logging.getLogger(__name__)


def main():
    """
    Run the Camply CLI. The CLI is wrapped in a main() function to
    interface with Poetry

    Returns
    ------
    None
    """
    logging.basicConfig(format="%(asctime)s [%(levelname)8s]: %(message)s",
                        level=CommandLineConfig.LOG_LEVEL)
    try:
        logger.camply(f"{CommandLineConfig.CAMPLY_DESCRIPTION} ⛺️")
        camply_cli = CamplyCommandLine()
        camply_cli.run_cli()
    except (KeyboardInterrupt, SystemExit):
        logger.debug("Handling Exit Request")
    finally:
        logger.camply(CommandLineConfig.CAMPLY_EXIT_MESSAGE)


class CommandLineError(Exception):
    """
    Generic CLI Error
    """
    pass


class CamplyCommandLine:
    """
    Camply Command Line Interface
    """

    __name__ = CommandLineConfig.CAMPLY_APP_NAME
    __version__ = CommandLineConfig.CAMPLY_APP_VERSION

    def __init__(self):
        """
        Initialized CLI
        """

        self.parser = ArgumentParser(description=CommandLineConfig.CAMPLY_LONG_DESCRIPTION,
                                     prog=self.__name__,
                                     epilog=CommandLineConfig.CAMPLY_EPILOG)
        self.arguments_compiled: bool = False
        self.arguments_parsed: bool = False
        self.cli_arguments: Optional[Namespace] = None
        self.arguments_validated: bool = False
        self.arguments_executed: bool = False

        self.recreation_areas = None
        self.campgrounds = None
        self.campsites = None

    def __repr__(self) -> str:
        """
        String Representation

        Returns
        -------
        str
        """
        return "<CamplyCommandLine>"

    def assemble_cli_arguments(self) -> None:
        """
        Gather the Arguments From the CLI

        Returns
        -------
        Namespace
        """
        assert self.arguments_compiled is False

        self.parser.add_argument(CommandLineConfig.VERSION_ARGUMENT,
                                 action=CommandLineConfig.VERSION,
                                 version=f"%(prog)s {self.__version__}")
        subparsers = self.parser.add_subparsers(dest=CommandLineConfig.COMMAND_DESTINATION)

        state_argument = ArgumentParser(add_help=False)
        state_argument.add_argument(CommandLineConfig.STATE_ARGUMENT,
                                    action=CommandLineConfig.STORE,
                                    dest=CommandLineConfig.STATE_DESTINATION,
                                    required=False,
                                    help=CommandLineConfig.STATE_HELP)

        recreation_area_id_argument = ArgumentParser(add_help=False)
        recreation_area_id_argument.add_argument(CommandLineConfig.REC_AREA_ID_ARGUMENT,
                                                 action=CommandLineConfig.APPEND,
                                                 dest=CommandLineConfig.REC_AREA_ID_DESTINATION,
                                                 required=False,
                                                 help=CommandLineConfig.REC_AREA_ID_HELP)

        campground_argument = ArgumentParser(add_help=False)
        campground_argument.add_argument(CommandLineConfig.CAMPGROUND_LIST_ARGUMENT,
                                         action=CommandLineConfig.APPEND,
                                         dest=CommandLineConfig.CAMPGROUND_LIST_DESTINATION,
                                         required=False,
                                         default=list(),
                                         help=CommandLineConfig.CAMPGROUND_LIST_HELP)

        search_argument = ArgumentParser(add_help=False)
        search_argument.add_argument(CommandLineConfig.SEARCH_ARGUMENT,
                                     action=CommandLineConfig.STORE,
                                     dest=CommandLineConfig.SEARCH_DESTINATION,
                                     required=False,
                                     help=CommandLineConfig.SEARCH_HELP)

        self.campsites = subparsers.add_parser(
            name=CommandLineConfig.COMMAND_CAMPSITES,
            help=CommandLineConfig.COMMAND_CAMPSITES_HELP,
            description=CommandLineConfig.COMMAND_CAMPSITES_DESCRIPTION,
            parents=[recreation_area_id_argument, campground_argument])

        self.recreation_areas = subparsers.add_parser(
            name=CommandLineConfig.COMMAND_RECREATION_AREA,
            help=CommandLineConfig.COMMAND_RECREATION_AREA_HELP,
            description=CommandLineConfig.COMMAND_RECREATION_AREA_DESCRIPTION,
            parents=[search_argument, state_argument])

        self.campgrounds = subparsers.add_parser(
            name=CommandLineConfig.COMMAND_CAMPGROUNDS,
            help=CommandLineConfig.COMMAND_CAMPGROUNDS_HELP,
            description=CommandLineConfig.COMMAND_CAMPGROUNDS_DESCRIPTION,
            parents=[search_argument, state_argument,
                     recreation_area_id_argument, campground_argument])

        subparsers.add_parser(
            name=CommandLineConfig.COMMAND_CONFIGURE,
            help=CommandLineConfig.COMMAND_CONFIGURE_HELP,
            description=CommandLineConfig.COMMAND_CONFIGURE_DESCRIPTION)

        self.campsites.add_argument(CommandLineConfig.START_DATE_ARGUMENT,
                                    action=CommandLineConfig.STORE,
                                    dest=CommandLineConfig.START_DATE_DESTINATION,
                                    required=False,
                                    help=CommandLineConfig.START_DATE_HELP)
        self.campsites.add_argument(CommandLineConfig.END_DATE_ARGUMENT,
                                    action=CommandLineConfig.STORE,
                                    dest=CommandLineConfig.END_DATE_DESTINATION,
                                    required=False,
                                    help=CommandLineConfig.END_DATE_HELP)
        self.campsites.add_argument(CommandLineConfig.WEEKENDS_ARGUMENT,
                                    action=CommandLineConfig.STORE_TRUE,
                                    dest=CommandLineConfig.WEEKENDS_DESTINATION,
                                    default=False,
                                    required=False,
                                    help=CommandLineConfig.WEEKENDS_HELP)
        self.campsites.add_argument(CommandLineConfig.NIGHTS_ARGUMENT,
                                    action=CommandLineConfig.STORE,
                                    dest=CommandLineConfig.NIGHTS_DESTINATION,
                                    default=CommandLineConfig.NIGHTS_DEFAULT,
                                    required=False,
                                    help=CommandLineConfig.NIGHTS_HELP)
        self.campsites.add_argument(CommandLineConfig.PROVIDER_ARGUMENT,
                                    action=CommandLineConfig.STORE,
                                    dest=CommandLineConfig.PROVIDER_DESTINATION,
                                    default=CommandLineConfig.PROVIDER_DEFAULT,
                                    required=False,
                                    help=CommandLineConfig.PROVIDER_HELP)
        self.campsites.add_argument(CommandLineConfig.CONTINUOUS_ARGUMENT,
                                    action=CommandLineConfig.STORE_TRUE,
                                    dest=CommandLineConfig.CONTINUOUS_DESTINATION,
                                    default=False,
                                    required=False,
                                    help=CommandLineConfig.CONTINUOUS_HELP)
        self.campsites.add_argument(CommandLineConfig.POLLING_INTERVAL_ARGUMENT,
                                    action=CommandLineConfig.STORE,
                                    dest=CommandLineConfig.POLLING_INTERVAL_DESTINATION,
                                    default=CommandLineConfig.POLLING_INTERVAL_DEFAULT,
                                    required=False,
                                    help=CommandLineConfig.POLLING_INTERVAL_HELP)
        self.campsites.add_argument(CommandLineConfig.NOTIFICATIONS_ARGUMENT,
                                    action=CommandLineConfig.STORE,
                                    dest=CommandLineConfig.NOTIFICATIONS_DESTINATION,
                                    required=False,
                                    default=CommandLineConfig.NOTIFICATIONS_DEFAULT,
                                    help=CommandLineConfig.NOTIFICATIONS_HELP)
        self.campsites.add_argument(CommandLineConfig.NOTIFY_FIRST_TRY_ARGUMENT,
                                    action=CommandLineConfig.STORE_TRUE,
                                    dest=CommandLineConfig.NOTIFY_FIRST_TRY_DESTINATION,
                                    required=False,
                                    default=False,
                                    help=CommandLineConfig.NOTIFY_FIRST_TRY_HELP)
        self.campsites.add_argument(CommandLineConfig.SEARCH_FOREVER_ARGUMENT,
                                    action=CommandLineConfig.STORE_TRUE,
                                    dest=CommandLineConfig.SEARCH_FOREVER_DESTINATION,
                                    required=False,
                                    default=False,
                                    help=CommandLineConfig.SEARCH_FOREVER_HELP)
        self.campsites.add_argument(CommandLineConfig.YAML_SEARCH_ARGUMENT,
                                    action=CommandLineConfig.STORE,
                                    dest=CommandLineConfig.YAML_SEARCH_DESTINATION,
                                    required=False,
                                    default=None,
                                    help=CommandLineConfig.YAML_SEARCH_HELP)
        self.arguments_compiled = True

    def parse_arguments(self) -> Namespace:
        """
        Actually Parse the Arguments

        Returns
        -------
        Namespace
        """
        assert self.arguments_compiled is True and self.arguments_parsed is False
        self.cli_arguments = self.parser.parse_args()
        self.arguments_parsed = True
        return self.cli_arguments

    def validate_arguments(self) -> bool:
        """
        Validate the Arguments Provided and Raise an Error if there's a problem

        Returns
        -------
        bool
        """
        assert self.arguments_validated is False
        error_message = None
        if self.cli_arguments.command is None:
            error_message = CommandLineConfig.ERROR_NO_ARGUMENT_FOUND
            help_parser = self.parser
        elif self.cli_arguments.command == CommandLineConfig.COMMAND_RECREATION_AREA:
            error_message, help_parser = self._validate_recreation_areas()
        elif self.cli_arguments.command == CommandLineConfig.COMMAND_CAMPGROUNDS:
            error_message, help_parser = self._validate_campgrounds()
        elif self.cli_arguments.command == CommandLineConfig.COMMAND_CAMPSITES:
            error_message, help_parser = self._validate_campsites()

        if error_message is not None:
            help_parser.print_help()
            print("\n")
            logger.error(error_message)
            exit(1)
        self.arguments_validated = True
        return self.arguments_validated

    def _validate_campsites(self) -> Tuple[Optional[str], Optional[ArgumentParser]]:
        """
        Validate the campsites portion of the CLI

        Returns
        -------
        Tuple[Optional[str], Optional[ArgumentParser]]:
        """
        error_message = help_parser = None
        if self.cli_arguments.provider == CommandLineConfig.RECREATION_DOT_GOV and all(
                [self.cli_arguments.recreation_area_id is None,
                 len(self.cli_arguments.campground_id) == 0,
                 self.cli_arguments.yml_config is None]):
            error_message = CommandLineConfig.ERROR_MESSAGE_REC_DOT_GOV
            help_parser = self.campsites
        mandatory_parameters = [self.cli_arguments.start_date,
                                self.cli_arguments.end_date]
        mandatory_string_parameters = [CommandLineConfig.START_DATE_ARGUMENT,
                                       CommandLineConfig.END_DATE_ARGUMENT]
        for field in mandatory_parameters:
            if field is None and self.cli_arguments.yml_config is None:
                error_message = (f"{CommandLineConfig.ERROR_MESSAGE_CAMPSITES}: "
                                 f"{', '.join(mandatory_string_parameters)}")
                help_parser = self.campsites
        return error_message, help_parser

    def _validate_campgrounds(self) -> Tuple[Optional[str], Optional[ArgumentParser]]:
        """
        Validate the campgrounds portion of the CLI

        Returns
        -------
        Tuple[Optional[str], Optional[ArgumentParser]]:
        """
        error_message = help_parser = None
        if all([self.cli_arguments.search is None,
                self.cli_arguments.state is None,
                self.cli_arguments.recreation_area_id is None,
                self.cli_arguments.campground_id is None]):
            error_message = CommandLineConfig.ERROR_MESSAGE_CAMPGROUNDS
            help_parser = self.campgrounds
        return error_message, help_parser

    def _validate_recreation_areas(self) -> Tuple[Optional[str], Optional[ArgumentParser]]:
        """
        Validate the recreation areas portion of the CLI

        Returns
        -------
        Tuple[Optional[str], Optional[ArgumentParser]]
        """
        error_message = help_parser = None
        if all([self.cli_arguments.search is None,
                self.cli_arguments.state is None]):
            error_message = CommandLineConfig.ERROR_MESSAGE_RECREATION_AREA
            help_parser = self.recreation_areas
        return error_message, help_parser

    def execute_cli_arguments(self) -> None:
        """
        Execute the CLI Arguments

        Returns
        -------
        None
        """
        assert self.arguments_validated is True
        assert self.arguments_executed is False

        if self.cli_arguments.command == CommandLineConfig.COMMAND_RECREATION_AREA:
            self.execute_recreation_areas()
        elif self.cli_arguments.command == CommandLineConfig.COMMAND_CAMPGROUNDS:
            self.execute_campgrounds()
        elif self.cli_arguments.command == CommandLineConfig.COMMAND_CAMPSITES:
            self.execute_campsites()
        elif self.cli_arguments.command == CommandLineConfig.COMMAND_CONFIGURE:
            configure_camply.generate_dot_camply_file()

    def execute_recreation_areas(self) -> None:
        """
        Execute the `recreation-areas` command on the CLI

        Returns
        -------
        None
        """
        camp_finder = RecreationDotGov()
        params = dict()
        if self.cli_arguments.state is not None:
            params.update(dict(state=self.cli_arguments.state))
        camp_finder.find_recreation_areas(search_string=self.cli_arguments.search,
                                          **params)

    def execute_campgrounds(self) -> None:
        """
        Execute the `recreation-areas` command on the CLI

        Returns
        -------
        None
        """
        camp_finder = RecreationDotGov()
        params = dict()
        if self.cli_arguments.state is not None:
            params.update(dict(state=self.cli_arguments.state))
        camp_finder.find_campgrounds(search_string=self.cli_arguments.search,
                                     rec_area_id=self.cli_arguments.recreation_area_id,
                                     campground_id=self.cli_arguments.campground_id,
                                     **params)

    def execute_campsites(self) -> None:
        """
        Execute the `recreation-areas` command on the CLI

        Returns
        -------
        None
        """
        if self.cli_arguments.yml_config is not None:
            provider, provider_kwargs, search_kwargs = yaml_utils.yaml_file_to_arguments(
                file_path=self.cli_arguments.yml_config)
        else:
            provider = self.cli_arguments.provider.lower()
            search_window = SearchWindow(
                start_date=datetime.strptime(self.cli_arguments.start_date, "%Y-%m-%d"),
                end_date=datetime.strptime(self.cli_arguments.end_date, "%Y-%m-%d"))
            provider_kwargs = dict(search_window=search_window,
                                   recreation_area=self.cli_arguments.recreation_area_id,
                                   campgrounds=self.cli_arguments.campground_id,
                                   weekends_only=self.cli_arguments.weekends,
                                   nights=int(self.cli_arguments.nights))
            search_kwargs = dict(log=True, verbose=True,
                                 continuous=self.cli_arguments.continuous,
                                 polling_interval=float(self.cli_arguments.polling_interval),
                                 notify_first_try=self.cli_arguments.notify_first_try,
                                 notification_provider=self.cli_arguments.notifications,
                                 search_forever=self.cli_arguments.search_forever)

        provider_class = {key.lower(): value for key, value in CAMPSITE_SEARCH_PROVIDER.items()}[
            provider.lower()]
        camping_finder = provider_class(**provider_kwargs)
        camping_finder.get_matching_campsites(**search_kwargs)

    def run_cli(self) -> None:
        """
        Full Execution of the CLI

        Returns
        -------
        None
        """
        self.assemble_cli_arguments()
        self.parse_arguments()
        self.validate_arguments()
        self.execute_cli_arguments()


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

# Author::    Justin Flannery  (mailto:juftin@juftin.com)

"""
Push Notifications via Pushover
"""

from abc import ABC
import base64
from datetime import datetime
import logging
from typing import List, Optional

import requests

from camply.config import CampsiteContainerFields, PushoverConfig
from .base_notifications import BaseNotifications
from ..containers import AvailableCampsite

logger = logging.getLogger(__name__)


class PushoverNotifications(BaseNotifications, logging.StreamHandler, ABC):
    """
    Push Notifications via Pushover + a Logging Handler
    """

    def __init__(self, level: Optional[int] = logging.INFO):
        logging.StreamHandler.__init__(self)
        self.setLevel(level=level)
        if any([PushoverConfig.PUSH_USER is None, PushoverConfig.PUSH_USER == ""]):
            warning_message = ("Pushover is not configured properly. To send pushover messages "
                               "make sure to run `camply configure` or set the "
                               "proper environment variables: `PUSHOVER_PUSH_USER`, "
                               "`PUSHOVER_PUSH_TOKEN`.")
            logger.error(warning_message)
            raise EnvironmentError(warning_message)

    def __repr__(self):
        return "<PushoverNotifications>"

    @staticmethod
    def send_message(message: str, **kwargs) -> Optional[requests.Response]:
        """
        Send a message via Pushover - if environment variables are configured

        Parameters
        ----------
        message: str

        Returns
        -------
        Response
        """
        token = PushoverConfig.PUSH_TOKEN if PushoverConfig.PUSH_TOKEN not in [None, ""] \
            else base64.b64decode(PushoverConfig.PUSHOVER_DEFAULT_API_TOKEN).decode("utf-8")
        response = requests.post(url=PushoverConfig.PUSHOVER_API_ENDPOINT,
                                 headers=PushoverConfig.API_HEADERS,
                                 params=dict(token=token,
                                             user=PushoverConfig.PUSH_USER,
                                             message=message,
                                             **kwargs)
                                 )
        if response.status_code != 200:
            logger.warning("Notifications weren't able to be sent to Pushover. "
                           "Your configuration might be incorrect.")
            raise ConnectionError(response.text)
        return response

    def emit(self, record: logging.LogRecord):
        """
        Produce a logging record

        Parameters
        ----------
        record: str
            Message to log
        """
        log_formatted_message = "[{:>10}]: {}".format(record.levelname.upper(),
                                                      record.msg)
        title = f"Pushover {record.levelname.title()} Message"
        self.send_message(message=log_formatted_message, title=title)

    @staticmethod
    def send_campsites(campsites: List[AvailableCampsite], **kwargs):
        """
        Send a message with a campsite object

        Parameters
        ----------
        campsites: AvailableCampsite
        """
        for campsite in campsites:
            fields = list()
            for key, value in campsite._asdict().items():
                if key == CampsiteContainerFields.BOOKING_URL:
                    key = "Booking Link"
                    value = f"<a href='{value}'>{value}"
                elif key in [CampsiteContainerFields.BOOKING_DATE,
                             CampsiteContainerFields.BOOKING_END_DATE]:
                    value: datetime = value.strftime("%Y-%m-%d")
                formatted_key = key.replace("_", " ").title()
                fields.append(f"<b>{formatted_key}:</b> {value}")
            composed_message = "\n".join(fields)
            message_title = (f"{campsite.recreation_area} | {campsite.facility_name} | "
                             f"{campsite.booking_date.strftime('%Y-%m-%d')}")
            PushoverNotifications.send_message(message=composed_message, title=message_title,
                                               html=1)



#!/usr/bin/env python3

# Author::    Justin Flannery  (mailto:juftin@juftin.com)

"""
Logging Utilities for Pushover Variables
"""
import logging

CALENDARMOJI = "📅"
CAMPMOJI = "🏕"
TENTMOJI = "⛺️"
XMOJI = "❌"


def get_emoji(obj: list) -> str:
    """
    Return the Right Emoji

    Parameters
    ----------
    obj: list

    Returns
    -------
    str
    """
    assert isinstance(obj, list)
    if len(obj) >= 1:
        return TENTMOJI
    else:
        return XMOJI


def log_camply(self: logging.Logger, message: str, *args, **kwargs) -> None:
    """
    Custom Logging Notification Level for Pushover Logging
    between logging.ERROR and logging.CRITICAL (45)
    Parameters
    ----------
    self: logging.Logger
    message: str
        Message String
    args
    kwargs
    Returns
    -------
    None
    """
    notification_level = logging.INFO + 1
    logging.addLevelName(level=notification_level, levelName="CAMPLY")
    if self.isEnabledFor(level=notification_level):
        self._log(level=notification_level, msg=message, args=args, **kwargs)



#!/usr/bin/env python3

# Author::    Justin Flannery  (mailto:juftin@juftin.com)

"""
Push Notifications via Pushbullet
"""

from abc import ABC
from datetime import datetime
import logging
from typing import List, Optional

import requests

from camply.config import CampsiteContainerFields, PushbulletConfig
from .base_notifications import BaseNotifications
from ..containers import AvailableCampsite

logger = logging.getLogger(__name__)


class PushbulletNotifications(BaseNotifications, ABC):
    """
    Push Notifications via PushBullet
    """

    def __init__(self):
        if any([PushbulletConfig.API_TOKEN is None,
                PushbulletConfig.API_TOKEN == ""]):
            warning_message = ("Pushbullet is not configured properly. To send Pushbullet messages "
                               "make sure to run `camply configure` or set the "
                               "proper environment variable: `PUSHBULLET_API_TOKEN`.")
            logger.error(warning_message)
            raise EnvironmentError(warning_message)

    def __repr__(self):
        return "<PushbulletNotifications>"

    @staticmethod
    def send_message(message: str, **kwargs) -> Optional[requests.Response]:
        """
        Send a message via PushBullet - if environment variables are configured
`
        Parameters
        ----------
        message: str

        Returns
        -------
        Response
        """
        pushbullet_headers = PushbulletConfig.API_HEADERS.copy()
        pushbullet_headers.update({"Access-Token": PushbulletConfig.API_TOKEN})
        message_type = kwargs.pop("type", "note")
        message_title = kwargs.pop("title", "Camply Notification")
        message_json = dict(type=message_type, title=message_title, body=message, **kwargs)
        logger.debug(message_json)
        response = requests.post(url=PushbulletConfig.PUSHBULLET_API_ENDPOINT,
                                 headers=pushbullet_headers,
                                 json=message_json)
        if response.status_code != 200:
            logger.warning("Notifications weren't able to be sent to Pushbullet. "
                           "Your configuration might be incorrect.")
            logger.debug(response.text)
        return response

    @staticmethod
    def send_campsites(campsites: List[AvailableCampsite], **kwargs):
        """
        Send a message with a campsite object

        Parameters
        ----------
        campsites: AvailableCampsite
        """
        for campsite in campsites:
            fields = list()
            for key, value in campsite._asdict().items():
                if key == CampsiteContainerFields.BOOKING_URL:
                    key = "booking_link"
                elif key in [CampsiteContainerFields.BOOKING_DATE,
                             CampsiteContainerFields.BOOKING_END_DATE]:
                    value: datetime = value.strftime("%Y-%m-%d")
                formatted_key = key.replace("_", " ").title()
                fields.append(f"{formatted_key}: {value}")
            composed_message = "\n".join(fields)
            message_title = (f"{campsite.recreation_area} | {campsite.facility_name} | "
                             f"{campsite.booking_date.strftime('%Y-%m-%d')}")
            PushbulletNotifications.send_message(message=composed_message, title=message_title,
                                                 type="note")
